////////////////////////////////////////////////////////////////
// File - WDREG.C
//
// Registers/Unregisters WinDriver in the Windows registery. 
// In order to install WinDriver in a new computer, you need 
// to copy WINDRVR.VXD file to C:\WIN95\SYSTEM\VMM32 (Win95)
// or copy WINDRVR.SYS file to C:\WINNT\SYSTEM32\DRIVERS
// (WinNT). After copying WinDriver to the drivers directory 
// you should add WinDriver to the list of Device Drivers Windows 
// loads on boot. This is done by "WDREG.EXE install".
// You can add this source to your own installation code, in 
// order to install WinDriver.
// 
////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>

void main (int argc, char *argv[])
{
    HKEY hKey;
    DWORD res;
    OSVERSIONINFO lVerInfo;
    DWORD val;
    int iInstall = 0; // 0-none, 1-install, 2-remove

    if (argc==2)
    {
        if (_stricmp(argv[1],"install")==0) iInstall = 1;
        if (_stricmp(argv[1],"remove")==0) iInstall = 2;
    }

    if (iInstall==0)
    {
        printf ("Usage: WDREG.EXE install\n");
        printf ("           addes WinDriver into registry\n");
        printf ("       WDREG.EXE remove\n");
        printf ("           removes WinDriver from registry\n");
        return;
    }

    lVerInfo.dwOSVersionInfoSize = sizeof (lVerInfo);
    if (!GetVersionEx (&lVerInfo)) 
    {
        printf ("Cannot get OS version\n");
        return;
    }

    switch (lVerInfo.dwPlatformId)
    {
        case VER_PLATFORM_WIN32s:
            printf ("This program can run only on Win95/WinNT");
            return;
            break;

        case VER_PLATFORM_WIN32_NT:
            if (iInstall==1)
            {
                printf ("Registering WINDRVR.SYS\n");
                //printf ("Copy WINDRVR.SYS to WINNT\\SYSTEM32\\DRIVERS directory\n");

                // updating registry
                RegCreateKeyEx (HKEY_LOCAL_MACHINE, "System\\CurrentControlSet\\Services\\WinDriver",0,"",0,KEY_ALL_ACCESS, NULL, &hKey, &res);
                if (res==REG_OPENED_EXISTING_KEY)
                {
                    printf ("WINDRVR.SYS already exists in Win-NT registry.\n");
                }

                val = 2;
                RegSetValueEx (hKey, "Start", 0, REG_DWORD, (PVOID) &val, 4);
                val = 1;
                RegSetValueEx (hKey, "Type", 0, REG_DWORD, (PVOID) &val, 4);
                val = 1;
                RegSetValueEx (hKey, "ErrorControl", 0, REG_DWORD, (PVOID) &val, 4);
                RegSetValueEx (hKey, "Group", 0, REG_SZ, "Extended Base", 14);
                RegSetValueEx (hKey, "ImagePath", 0, REG_SZ, "\\SystemRoot\\System32\\drivers\\windrvr.sys", 41);
                RegSetValueEx (hKey, "DisplayName", 0, REG_SZ, "WinDriver", 10);

                RegCloseKey (hKey);
            }
            else
            {
                printf ("UnRegistering WINDRVR.SYS\n");
                RegDeleteKey (HKEY_LOCAL_MACHINE, "System\\CurrentControlSet\\Services\\WinDriver");
                RegCloseKey (hKey);
            }
            break;

        case VER_PLATFORM_WIN32_WINDOWS:
            if (iInstall==1)
            {
                printf ("Registering WINDRVR.VXD \n");
                //printf ("Copy WINDRVR.VXD to WIN95\\SYSTEM\\VMM32 directory\n");

                // updating registry
                RegCreateKeyEx (HKEY_LOCAL_MACHINE, "System\\CurrentControlSet\\Services\\VxD\\WinDriver",0,"",0,KEY_ALL_ACCESS, NULL, &hKey, &res);
                if (res==REG_OPENED_EXISTING_KEY)
                {
                    printf ("WINDRVR.VXD already exists in Win-95 registry.\n");
                }

                RegSetValueEx (hKey, "Start", 0, REG_BINARY, "\0", 1);
                RegSetValueEx (hKey, "StaticVxD", 0, REG_SZ, "*windrvr", 9);

                RegCloseKey (hKey);
            }
            else
            {
                printf ("UnRegistering WINDRVR.VXD\n");
                RegDeleteKey (HKEY_LOCAL_MACHINE, "System\\CurrentControlSet\\Services\\VxD\\WinDriver");
                RegCloseKey (hKey);
            }

            break;

        default:
            printf ("This program can run only on Win95/WinNT");
            return;
            break;
    }    
}
